@@ -3,8 +3,8 @@
  * Copyright (C) 2012-2016 Sebastien Jodogne, Medical Physics
  * Department, University Hospital of Liege, Belgium
  * Copyright (C) 2017-2023 Osimis S.A., Belgium
- * Copyright (C) 2024-2024 Orthanc Team SRL, Belgium
- * Copyright (C) 2021-2024 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
+ * Copyright (C) 2024-2025 Orthanc Team SRL, Belgium
+ * Copyright (C) 2021-2025 Sebastien Jodogne, ICTEAM UCLouvain, Belgium
  *
  * This program is free software: you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -33,6 +33,8 @@
 #endif
 
 #include "IDicomTranscoder.h"
+#include "../MultiThreading/Semaphore.h"
+
 
 namespace Orthanc
 {
@@ -40,7 +42,8 @@ namespace Orthanc
   {
   private:
     unsigned int  lossyQuality_;
-    
+    Semaphore maxConcurrentExecutionsSemaphore_;
+
     bool InplaceTranscode(DicomTransferSyntax& selectedSyntax /* out */,
                           std::string& failureReason /* out */,
                           DcmFileFormat& dicom,
@@ -48,7 +51,7 @@ namespace Orthanc
                           bool allowNewSopInstanceUid);
     
   public:
-    DcmtkTranscoder();
+    explicit DcmtkTranscoder(unsigned int maxConcurrentExecutions);
 
     void SetLossyQuality(unsigned int quality);
 
